/**
 * Negatives Formatter - Windows + Linux
 * Originally Developed By: Takashi Binns @ Sisense
 * Updated and Adapted by: Ben Richie Sadan @ Sisense
 * Version : 2.0
 * Last Modified Date : 31-Jan-2020
 */

//	Initialize the copyWidget object
prism.run(["$filter", function ($filter) {

    //////////////////////////
    //	Plugin properties	//
    //////////////////////////

    //	Define allowed chart types
    var supportedChartTypes = ["pivot", "pivot2", "indicator"];

    var chartType;

    //  Define properties    
    var labelHeader = "Financial Formatting";
    var options = {
        "Format Negatives": false,
        "Show Zeros": false
    };
    var labelProperty = 'FinancialFormatting';


    //////////////////////////
    //	Utility Functions	//
    //////////////////////////

    //	Function to determine if this chart type is supported
    function widgetIsSupported(type) {
        if (supportedChartTypes.indexOf(type) >= 0) {
            return true;
        } else {
            return false;
        }
    }

    //////////////////////////////////
    //  Main Functions              //
    //////////////////////////////////

    //  Define formatter function (type can be 'percent' or 'currency')
    function valueFormatter(element, type, labelText, widget) {

        //  Adjust the color
        $(element).css('color', '#ff3737');

        if (labelText && labelText.length > 0) {
            //  Adjust the text        
            var newText = '';
            if (type == 'currency') {
                newText = labelText.replace("$-", "($") + ")";
            } else {
                newText = labelText.replace("-", "(") + ")";
            }

            if (widget.type == "pivot") {
                $(element).text(newText);
            } else if (widget.type == "pivot2") {
                $('.table-grid__content__inner', element)[0].innerText = newText;
            } else {
                return newText;
            }
        }
    }

    //  Function to format pivot tables
    function pivotFormatter(widget, event) {

        //  Get the widget element
        var widgetElement = (prism.$ngscope.appstate == 'dashboard') ? $('widget[widgetid="' + widget.oid + '"]') : $('pivot');

        //  Make sure there's at least some default setttings
        if (typeof widget.options[labelProperty] === "undefined") {
            widget.options[labelProperty] = $.extend({}, options);
        }

        //  loop through the pivot table's value metadata items
        var valueItems = widget.metadata.panel("values").items;
        $.each(valueItems, function () {

            //  Get the fidx
            var fidx = this.field.index;

            //  Is this value formatted as a currency or percent?
            var isCurrency = (this.format && this.format.mask && this.format.mask.currency),
                isPercent = (this.format && this.format.mask && this.format.mask.percent);

            //  Run the code to show zeros instead of blank cells
            if (widget.options[labelProperty]["Show Zeros"]) {

                var cells;

                if (widget.type == "pivot") {
                    //  Find all the table cells, that have a null value
                    cells = $('td[fidx=' + fidx + ']:has(.p-value)', widgetElement).filter(function () {
                        return $(this).attr('val') === "N\\A";
                    })
                } else if (widget.type == "pivot2") {
                    cells = $('.table-grid__cell', widgetElement).filter(function () {
                        var val = this.innerText.replace('$', '');
                        val = val.replace(',', '');
                        return parseFloat(val) === 0;
                    })
                }

                //  Determine the new text to include
                var newText = "0";
                if (isCurrency) {
                    newText = "$0";
                } else if (isPercent) {
                    newText = "0%";
                }

                //  Write in a zero for these cells
                if (widget.type == "pivot") {
                    cells.text(newText);
                } else if (widget.type == "pivot2") {
                    $.each(cells, function () {
                        $('.table-grid__content__inner', element)[0].innerText = newText;
                    })
                }
            }

            //  Run the code to format data values
            if (widget.options[labelProperty]["Format Negatives"]) {
                //  Only applies to currency and percentage formatting
                if (isCurrency || isPercent) {
                    var cells;

                    if (widget.type == "pivot") {
                        //  Find all the table cells, that have a null value
                        cells = $('td[fidx=' + fidx + ']:has(.p-value)', widgetElement)
                    } else if (widget.type == "pivot2") {
                        cells = $('.table-grid__cell', widgetElement);
                    }

                    //  Get a series type
                    var sType = isCurrency ? 'currency' : 'percent';

                    //  Loop through each cell and run the formatter
                    $.each(cells, function () {
                        var shouldFormat = false;

                        if (widget.type == "pivot") {
                            shouldFormat = ($(this).attr('val') < 0)
                        } else if (widget.type == "pivot2") {
                            var val = this.innerText.replace('$', '');
                            val = val.replace('%', '');
                            val = val.replace(',', '');

                            shouldFormat = (parseFloat(val) < 0);
                        }

                        if (shouldFormat) {
                            valueFormatter(this, sType, $(this).text(), widget);
                        } else {
                            $(this).css('color', 'black');
                        }
                    })
                }
            }
        })
    }

    //  Function to format indicators
    function indicatorFormatter(widget, event) {

        //  Find the metadata items for this widget
        var mainValueItem = widget.metadata.panel("value").items.length > 0 ? widget.metadata.panel("value").items[0] : null;
        //secondValueItem = widget.metadata.panel("secondary").items.length > 0 ? widget.metadata.panel("secondary").items[0] : null;

        if (mainValueItem != null) {
            handleIndicatorValue(widget, mainValueItem, widget.queryResult.value);

            widget.queryResult.gauge.color = mainValueItem.format.color.color;
        }

        // if (secondValueItem != null) {
        //     secondValueItem.format.color = {};

        //     handleIndicatorValue(widget, secondValueItem, widget.queryResult.secondary);
        // }
    }

    function handleIndicatorValue(widget, valuePanel, dataObject) {

        //  Is this value formatted as a currency or percent?
        var isCurrency = (valuePanel.format && valuePanel.format.mask && valuePanel.format.mask.currency),
            isPercent = (valuePanel.format && valuePanel.format.mask && valuePanel.format.mask.percent);

        //  Run the code to show zeros instead of blank cells
        if (widget.options[labelProperty]["Show Zeros"]) {

            //  Find the secondary number
            if (dataObject.text.includes('N/A')) {
                //  Determine the new text to include
                var newText = "0";
                if (isCurrency) {
                    newText = "$0";
                } else if (isPercent) {
                    newText = "0%";
                }

                dataObject.text = newText;
            }
        }

        //  Run the code to format data values
        var valueIsNegative = (dataObject.data < 0);
        if (valueIsNegative && widget.options[labelProperty]["Format Negatives"]) {

            //  Only applies to currency and percentage formatting
            if (isCurrency || isPercent) {

                //  Get a series type
                var sType = isCurrency ? 'currency' : 'percent';

                //  Loop through each cell and run the formatter
                dataObject.text = valueFormatter(null, sType, dataObject.text, widget);

                valuePanel.format.color.color = '#ff3737';
                //dataObject.color = '#ff3737';
            }
        } else {
            //$(this).css('color', '#000000');

            valuePanel.format.color.color = '#000000';
            //dataObject.color = '#000000';
        }
    }

    //////////////////////////////////
    //  Initialization Functions    //
    //////////////////////////////////

    // Registering dashboard/ widget creation in order to perform drilling
    prism.on("dashboardloaded", function (e, args) {
        args.dashboard.on("widgetinitialized", onWidgetInitialized);
    });

    // register widget events upon initialization
    function onWidgetInitialized(dashboard, args) {

        //	Hooking to ready/destroyed events
        args.widget.on("destroyed", onWidgetDestroyed);

        //	Should we run this code
        var shouldInit = widgetIsSupported(args.widget.type);

        //	Add hook on render
        if (shouldInit) {

            //  Use different handling function depending on the widget type
            if (args.widget.type == "pivot" || args.widget.type == "pivot2") {
                addHandler("ready", pivotFormatter, args.widget);
            } else if (args.widget.type == "indicator") {
                addHandler("render", indicatorFormatter, args.widget);
            }
        }
    }

    // unregistering widget events
    function onWidgetDestroyed(widget, args) {
        widget.off("destroyed", onWidgetDestroyed);
    }

    //  Add handler function
    function addHandler(event, myFunction, widget) {

        //  Look for handlers for this event
        var existingHandlers = widget.$$events[event] ? widget.$$events[event].handlers : null;

        //  Only run if the event was found
        if (existingHandlers) {
            //  Only run if the handler has already been added to this functino
            if (existingHandlers.indexOf(myFunction) < 0) {
                //  Add a new handler
                existingHandlers.push(myFunction);
            }
        }
    }

    //////////////////////////////////
    //  Menu  Functions             //
    //////////////////////////////////

    //  Function to make sure this is a settings menu
    function canShowMenu(e, args) {

        //  Assume true by default
        var result = true;
        try {

            //  Only show this in the widget editor
            if (prism.$ngscope.appstate == 'dashboard') {
                result = false;
            }

            // find the widget
            var widget = e.currentScope.widget;
            if (widget == undefined) {
                result = false;
            }

            // Widget must be an allowed chart
            if (!widgetIsSupported(widget.type)) {
                result = false;
            }

            //  Make sure the user clicked on a settings menu
            if (args.ui.css !== "w-menu-host") {
                result = false;
            }
        } catch (e) {
            result = false;
        }

        //  Return result
        return result;
    }

    //	Create Options for the menu
    prism.on("beforemenu", function (e, args) {

        //  Should the new menu item be added?
        var shouldAdd = canShowMenu(e, args);
        if (shouldAdd) {

            //  Get the widget
            var widget = e.currentScope.widget;

            //  Get saved setting
            var settings = widget.options[labelProperty];
            if (typeof settings === 'undefined') {
                //  Create object with defaults, if no settings defined
                settings = $.extend({}, options);
            }

            //  Function for when the item is clicked
            var itemClicked = function () {
                //  Update the axis setting
                this.settings[this.caption] = !this.settings[this.caption];
                widget.options[labelProperty] = this.settings;
                //  Redraw the widget
                widget.redraw();
            }

            //  Create and add header
            var header = {
                caption: labelHeader,
                type: "header"
            };
            args.settings.items.push(header);

            //  Loop through all the options and add to the menu
            for (key in settings) {

                //  Create menu item                
                var menuItem = {
                    caption: key,
                    checked: settings[key],
                    size: 'xl',
                    type: 'check',
                    closing: false,
                    widget: widget,
                    settings: settings,
                    execute: itemClicked
                };

                //  Add to the menu
                args.settings.items.push(menuItem);
            }
        }
    });
}]);